# pylint: disable=W0621
"""Asynchronous Python client for the P1 Monitor API."""

import asyncio

from p1monitor import P1Monitor, WaterMeter


async def main() -> None:
    """Show example on getting P1 Monitor data."""
    async with P1Monitor(host="127.0.0.1") as client:
        watermeter: WaterMeter = await client.watermeter()

        print(watermeter)
        print()
        print("--- P1 Monitor | WaterMeter ---")
        print(f"Consumption Day: {watermeter.consumption_day}")
        print(f"Consumption Total: {watermeter.consumption_total}")
        print(f"Pulse Count: {watermeter.pulse_count}")


if __name__ == "__main__":
    asyncio.run(main())



"""Exceptions tests for the P1Monitor device."""

# pylint: disable=protected-access
import pytest
from aresponses import ResponsesMockServer

from p1monitor import P1Monitor
from p1monitor.exceptions import P1MonitorConnectionError, P1MonitorError


@pytest.mark.parametrize("status", [401, 403])
async def test_http_error401(
    aresponses: ResponsesMockServer,
    p1monitor_client: P1Monitor,
    status: int,
) -> None:
    """Test HTTP 401 response handling."""
    aresponses.add(
        "192.168.1.2",
        "/api/v1/smartmeter",
        "GET",
        aresponses.Response(text="Give me energy!", status=status),
    )
    with pytest.raises(P1MonitorConnectionError):
        assert await p1monitor_client._request("test")


async def test_http_error404(
    aresponses: ResponsesMockServer,
    p1monitor_client: P1Monitor,
) -> None:
    """Test HTTP 404 response handling."""
    aresponses.add(
        "192.168.1.2",
        "/api/v1/smartmeter",
        "GET",
        aresponses.Response(text="Give me energy!", status=404),
    )
    with pytest.raises(P1MonitorError):
        assert await p1monitor_client._request("test")


async def test_http_error500(
    aresponses: ResponsesMockServer,
    p1monitor_client: P1Monitor,
) -> None:
    """Test HTTP 500 response handling."""
    aresponses.add(
        "192.168.1.2",
        "/api/v1/smartmeter",
        "GET",
        aresponses.Response(
            body=b'{"status":"nok"}',
            status=500,
        ),
    )
    with pytest.raises(P1MonitorError):
        assert await p1monitor_client._request("test")


async def test_no_success(
    aresponses: ResponsesMockServer,
    p1monitor_client: P1Monitor,
) -> None:
    """Test a message without a success message throws."""
    aresponses.add(
        "192.168.1.2",
        "/api/v1/smartmeter",
        "GET",
        aresponses.Response(
            status=200,
            text='{"message": "no success"}',
        ),
    )
    with pytest.raises(P1MonitorError):
        assert await p1monitor_client._request("test")



"""Fixture for the P1Monitor tests."""

from collections.abc import AsyncGenerator

import pytest
from aiohttp import ClientSession

from p1monitor import P1Monitor


@pytest.fixture(name="p1monitor_client")
async def client() -> AsyncGenerator[P1Monitor, None]:
    """Return a P1Monitor client."""
    async with (
        ClientSession() as session,
        P1Monitor(host="192.168.1.2", port=80, session=session) as p1monitor_client,
    ):
        yield p1monitor_client



# pylint: disable=W0621
"""Asynchronous Python client for the P1 Monitor API."""

import asyncio

from p1monitor import P1Monitor, Phases


async def main() -> None:
    """Show example on getting P1 Monitor data."""
    async with P1Monitor(host="127.0.0.1") as client:
        phases: Phases = await client.phases()

        print(phases)
        print()
        print("--- P1 Monitor | Phases ---")
        print(f"Voltage Phase L1: {phases.voltage_phase_l1}")
        print(f"Voltage Phase L2: {phases.voltage_phase_l2}")
        print(f"Voltage Phase L3: {phases.voltage_phase_l3}")
        print()
        print(f"Current Phase L1: {phases.current_phase_l1}")
        print(f"Current Phase L2: {phases.current_phase_l2}")
        print(f"Current Phase L3: {phases.current_phase_l3}")
        print()
        print(f"Power Consumed Phase L1: {phases.power_consumed_phase_l1}")
        print(f"Power Consumed Phase L2: {phases.power_consumed_phase_l2}")
        print(f"Power Consumed Phase L3: {phases.power_consumed_phase_l3}")
        print()
        print(f"Power Produced Phase L1: {phases.power_produced_phase_l1}")
        print(f"Power Produced Phase L2: {phases.power_produced_phase_l2}")
        print(f"Power Produced Phase L3: {phases.power_produced_phase_l3}")


if __name__ == "__main__":
    asyncio.run(main())



"""Asynchronous Python client for the P1 Monitor API."""

from .exceptions import P1MonitorConnectionError, P1MonitorError, P1MonitorNoDataError
from .models import Phases, Settings, SmartMeter, WaterMeter
from .p1monitor import P1Monitor

__all__ = [
    "P1Monitor",
    "P1MonitorConnectionError",
    "P1MonitorError",
    "P1MonitorNoDataError",
    "Phases",
    "Settings",
    "SmartMeter",
    "WaterMeter",
]



# pylint: disable=W0621
"""Asynchronous Python client for the P1 Monitor API."""

import asyncio

from p1monitor import P1Monitor, Settings


async def main() -> None:
    """Show example on getting P1 Monitor data."""
    async with P1Monitor(host="127.0.0.1") as client:
        settings: Settings = await client.settings()

        print(settings)
        energy_cons_high = settings.energy_consumption_price_high
        energy_cons_low = settings.energy_consumption_price_low
        energy_prod_high = settings.energy_production_price_high
        energy_prod_low = settings.energy_production_price_low
        print()
        print("--- P1 Monitor | Settings ---")
        print(f"Gas Price: {settings.gas_consumption_price}")
        print(f"Energy Consumption Price - High: {energy_cons_high}")
        print(f"Energy Consumption Price - Low: {energy_cons_low}")
        print(f"Energy Production Price - High: {energy_prod_high}")
        print(f"Energy Production Price - Low: {energy_prod_low}")


if __name__ == "__main__":
    asyncio.run(main())



# pylint: disable=W0621
"""Asynchronous Python client for the P1 Monitor API."""

import asyncio

from p1monitor import P1Monitor, SmartMeter


async def main() -> None:
    """Show example on getting P1 Monitor data."""
    async with P1Monitor(host="127.0.0.1") as client:
        smartmeter: SmartMeter = await client.smartmeter()

        print(smartmeter)
        print()
        print("--- P1 Monitor | SmartMeter ---")
        print(f"Power Consumption: {smartmeter.power_consumption}")
        print(f"Energy Consumption - High: {smartmeter.energy_consumption_high}")
        print(f"Energy Consumption - Low: {smartmeter.energy_consumption_low}")
        print()
        print(f"Power Production: {smartmeter.power_production}")
        print(f"Energy Production - High: {smartmeter.energy_production_high}")
        print(f"Energy Production - Low: {smartmeter.energy_production_low}")
        print(f"Energy Tariff: {smartmeter.energy_tariff_period}")
        print()
        print(f"Gas Consumption: {smartmeter.gas_consumption}")


if __name__ == "__main__":
    asyncio.run(main())



"""Models for P1 Monitor."""

from __future__ import annotations

from dataclasses import dataclass
from enum import Enum
from typing import Any


class EnergyTariff(str, Enum):
    """Enumeration representing the rate period."""

    __slots__ = ()

    LOW = "low"
    HIGH = "high"


@dataclass
class SmartMeter:
    """Object representing an SmartMeter response from P1 Monitor."""

    gas_consumption: float | None
    energy_tariff_period: str | None

    power_consumption: int | None
    energy_consumption_high: float | None
    energy_consumption_low: float | None

    power_production: int | None
    energy_production_high: float | None
    energy_production_low: float | None

    @staticmethod
    def from_dict(data: dict[str | int, Any]) -> SmartMeter:
        """Return SmartMeter object from the P1 Monitor API response.

        Args:
        ----
            data: The data from the P1 Monitor API.

        Returns:
        -------
            A SmartMeter object.

        """

        def energy_tariff(tariff: str) -> EnergyTariff:
            """Return API energy_tariff information.

            Args:
            ----
                tariff: The provided tariff code from the API.

            Returns:
            -------
                The energy tariff period class.

            """
            if tariff == "P":
                return EnergyTariff.HIGH
            return EnergyTariff.LOW

        data = data[0]
        return SmartMeter(
            gas_consumption=data.get("CONSUMPTION_GAS_M3"),
            power_consumption=data.get("CONSUMPTION_W"),
            energy_consumption_high=data.get("CONSUMPTION_KWH_HIGH"),
            energy_consumption_low=data.get("CONSUMPTION_KWH_LOW"),
            power_production=data.get("PRODUCTION_W"),
            energy_production_high=data.get("PRODUCTION_KWH_HIGH"),
            energy_production_low=data.get("PRODUCTION_KWH_LOW"),
            energy_tariff_period=energy_tariff(str(data.get("TARIFCODE"))),
        )


@dataclass
class Settings:
    """Object representing an Settings response from P1 Monitor."""

    gas_consumption_price: float | None

    energy_consumption_price_high: float | None
    energy_consumption_price_low: float | None

    energy_production_price_high: float | None
    energy_production_price_low: float | None

    @staticmethod
    def from_dict(data: dict[str, Any]) -> Settings:
        """Return Settings object from the P1 Monitor API response.

        Args:
        ----
            data: The data from the P1 Monitor API.

        Returns:
        -------
            A Settings object.

        """
        return Settings(
            gas_consumption_price=search(15, data, "conf"),
            energy_consumption_price_low=search(1, data, "conf"),
            energy_consumption_price_high=search(2, data, "conf"),
            energy_production_price_low=search(3, data, "conf"),
            energy_production_price_high=search(4, data, "conf"),
        )


@dataclass
class Phases:
    """Object representing an Phases response from P1 Monitor."""

    voltage_phase_l1: float | None
    voltage_phase_l2: float | None
    voltage_phase_l3: float | None

    current_phase_l1: float | None
    current_phase_l2: float | None
    current_phase_l3: float | None

    power_consumed_phase_l1: int | None
    power_consumed_phase_l2: int | None
    power_consumed_phase_l3: int | None

    power_produced_phase_l1: int | None
    power_produced_phase_l2: int | None
    power_produced_phase_l3: int | None

    @staticmethod
    def from_dict(data: dict[str | int, Any]) -> Phases:
        """Return Phases object from the P1 Monitor API response.

        Args:
        ----
            data: The data from the P1 Monitor API.

        Returns:
        -------
            A Phases object.

        """
        return Phases(
            voltage_phase_l1=search(103, data, "status"),
            voltage_phase_l2=search(104, data, "status"),
            voltage_phase_l3=search(105, data, "status"),
            current_phase_l1=search(100, data, "status"),
            current_phase_l2=search(101, data, "status"),
            current_phase_l3=search(102, data, "status"),
            power_consumed_phase_l1=convert(search(74, data, "status")),
            power_consumed_phase_l2=convert(search(75, data, "status")),
            power_consumed_phase_l3=convert(search(76, data, "status")),
            power_produced_phase_l1=convert(search(77, data, "status")),
            power_produced_phase_l2=convert(search(78, data, "status")),
            power_produced_phase_l3=convert(search(79, data, "status")),
        )


@dataclass
class WaterMeter:
    """Object representing an WaterMeter response from P1 Monitor."""

    consumption_day: int | None
    consumption_total: float | None
    pulse_count: int | None

    @staticmethod
    def from_dict(data: dict[str | int, Any]) -> WaterMeter:
        """Return WaterMeter object from the P1 Monitor API response.

        Args:
        ----
            data: The data from the P1 Monitor API.

        Returns:
        -------
            A WaterMeter object.

        """
        data = data[0]
        return WaterMeter(
            consumption_day=data.get("WATERMETER_CONSUMPTION_LITER"),
            consumption_total=data.get("WATERMETER_CONSUMPTION_TOTAL_M3"),
            pulse_count=data.get("WATERMETER_PULS_COUNT"),
        )


def search(position: int, data: Any, service: str) -> float:
    """Find the correct value in the json data file.

    Args:
    ----
        position: The position ID number.
        data: The JSON list which is requested from the API.
        service: Type of dataclass.

    Returns:
    -------
        The value that corresponds to the specified position.

    """
    value: float
    for i in data:
        if service == "conf" and i["CONFIGURATION_ID"] == position:
            value = float(i["PARAMETER"])
        elif service == "status" and i["STATUS_ID"] == position:
            value = float(i["STATUS"])
    return value


def convert(value: float) -> int:
    """Convert values from kW to W.

    Args:
    ----
        value: The current value.

    Returns:
    -------
        Value in Watt (W).

    """
    return int(float(value) * 1000)



"""Exceptions for P1 Monitor."""


class P1MonitorError(Exception):
    """Generic P1 Monitor exception."""


class P1MonitorConnectionError(P1MonitorError):
    """P1 Monitor connection exception."""


class P1MonitorNoDataError(P1MonitorError):
    """P1 Monitor no data exception."""



"""Basic tests for the P1Monitor device."""

# pylint: disable=protected-access
import asyncio
from unittest.mock import patch

import pytest
from aiohttp import ClientError, ClientResponse, ClientSession
from aresponses import Response, ResponsesMockServer

from p1monitor import P1Monitor
from p1monitor.exceptions import P1MonitorConnectionError, P1MonitorError

from . import load_fixtures


async def test_json_request(
    aresponses: ResponsesMockServer,
    p1monitor_client: P1Monitor,
) -> None:
    """Test JSON response is handled correctly."""
    aresponses.add(
        "192.168.1.2",
        "/api/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        ),
    )
    response = await p1monitor_client._request("test")
    assert response is not None
    await p1monitor_client.close()


async def test_different_port(aresponses: ResponsesMockServer) -> None:
    """Test different port is handled correctly."""
    aresponses.add(
        "192.168.1.2:84",
        "/api/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        ),
    )
    async with ClientSession() as session:
        p1monitor = P1Monitor("192.168.1.2", port=84, session=session)
        await p1monitor._request("test")
        await p1monitor.close()


async def test_internal_session(aresponses: ResponsesMockServer) -> None:
    """Test JSON response is handled correctly."""
    aresponses.add(
        "192.168.1.2",
        "/api/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "application/json"},
            text='{"status": "ok"}',
        ),
    )
    async with P1Monitor("192.168.1.2") as p1monitor:
        await p1monitor._request("test")


async def test_timeout(aresponses: ResponsesMockServer) -> None:
    """Test request timeout from P1 Monitor."""

    # Faking a timeout by sleeping
    async def reponse_handler(_: ClientResponse) -> Response:
        await asyncio.sleep(0.2)
        return aresponses.Response(
            body="Goodmorning!",
            text=load_fixtures("smartmeter.json"),
        )

    aresponses.add("192.168.1.2", "/api/test", "GET", reponse_handler)

    async with ClientSession() as session:
        client = P1Monitor(host="192.168.1.2", session=session, request_timeout=0.1)
        with pytest.raises(P1MonitorConnectionError):
            assert await client._request("test")


async def test_content_type(
    aresponses: ResponsesMockServer,
    p1monitor_client: P1Monitor,
) -> None:
    """Test request content type error from P1 Monitor."""
    aresponses.add(
        "192.168.1.2",
        "/api/test",
        "GET",
        aresponses.Response(
            status=200,
            headers={"Content-Type": "blabla/blabla"},
        ),
    )
    with pytest.raises(P1MonitorError):
        assert await p1monitor_client._request("test")


async def test_client_error() -> None:
    """Test request client error from P1 Monitor."""
    async with ClientSession() as session:
        client = P1Monitor(host="192.168.1.2", session=session)
        with (
            patch.object(
                session,
                "request",
                side_effect=ClientError,
            ),
            pytest.raises(P1MonitorConnectionError),
        ):
            assert await client._request("test")



"""Asynchronous Python client for the P1 Monitor API."""

from __future__ import annotations

import asyncio
import socket
from dataclasses import dataclass
from importlib import metadata
from typing import Any, Self

from aiohttp import ClientError, ClientSession
from aiohttp.hdrs import METH_GET
from yarl import URL

from .exceptions import P1MonitorConnectionError, P1MonitorError, P1MonitorNoDataError
from .models import Phases, Settings, SmartMeter, WaterMeter

VERSION = metadata.version(__package__)


@dataclass
class P1Monitor:
    """Main class for handling connections with the P1 Monitor API."""

    host: str
    port: int = 80
    request_timeout: float = 10.0
    session: ClientSession | None = None

    _close_session: bool = False

    async def _request(
        self,
        uri: str,
        *,
        method: str = METH_GET,
        params: dict[str, Any] | None = None,
    ) -> Any:
        """Handle a request to a P1 Monitor device.

        Args:
        ----
            uri: Request URI, without '/api/', for example, 'status'
            method: HTTP Method to use.
            params: Extra options to improve or limit the response.

        Returns:
        -------
            A Python dictionary (JSON decoded) with the response from
            the P1 Monitor API.

        Raises:
        ------
            P1MonitorConnectionError: An error occurred while communicating
                with the P1 Monitor.
            P1MonitorError: Received an unexpected response from the P1 Monitor API.

        """
        url = URL.build(
            scheme="http",
            host=self.host,
            port=int(self.port),
            path="/api/",
        ).join(URL(uri))

        headers = {
            "User-Agent": f"PythonP1Monitor/{VERSION}",
            "Accept": "application/json, text/plain, */*",
        }

        if self.session is None:
            self.session = ClientSession()
            self._close_session = True

        try:
            async with asyncio.timeout(self.request_timeout):
                response = await self.session.request(
                    method,
                    url,
                    params=params,
                    headers=headers,
                )
                response.raise_for_status()

        except TimeoutError as exception:
            msg = "Timeout occurred while connecting to P1 Monitor device"
            raise P1MonitorConnectionError(
                msg,
            ) from exception
        except (ClientError, socket.gaierror) as exception:
            if "watermeter" in uri and response.status == 404:
                msg = "No water meter is connected to P1 Monitor device"
                raise P1MonitorConnectionError(msg) from exception
            msg = "Error occurred while communicating with P1 Monitor device"
            raise P1MonitorConnectionError(msg) from exception

        content_type = response.headers.get("Content-Type", "")
        if "application/json" not in content_type:
            text = await response.text()
            msg = "Unexpected response from the P1 Monitor device"
            raise P1MonitorError(
                msg,
                {"Content-Type": content_type, "response": text},
            )

        return await response.json()

    async def smartmeter(self) -> SmartMeter:
        """Get the latest values from you smart meter.

        Returns
        -------
            A SmartMeter data object from the P1 Monitor API.

        """
        data = await self._request(
            "v1/smartmeter",
            params={"json": "object", "limit": 1},
        )
        return SmartMeter.from_dict(data)

    async def settings(self) -> Settings:
        """Receive the set price values for energy and gas.

        Returns
        -------
            A Settings data object from the P1 Monitor API.

        """
        data = await self._request("v1/configuration", params={"json": "object"})
        return Settings.from_dict(data)

    async def phases(self) -> Phases:
        """Receive data from all phases on your smart meter.

        Returns
        -------
            A Phases data object from the P1 Monitor API.

        """
        data = await self._request("v1/status", params={"json": "object"})
        return Phases.from_dict(data)

    async def watermeter(self) -> WaterMeter:
        """Get the latest values from you water meter.

        Returns
        -------
            A WaterMeter data object from the P1 Monitor API.

        Raises
        ------
            P1MonitorNoDataError: No data was received from the P1 Monitor API.

        """
        data = await self._request(
            "v2/watermeter/day",
            params={"json": "object", "limit": 1},
        )
        if data == []:
            msg = "No data received from P1 Monitor"
            raise P1MonitorNoDataError(msg)
        return WaterMeter.from_dict(data)

    async def close(self) -> None:
        """Close open client session."""
        if self.session and self._close_session:
            await self.session.close()

    async def __aenter__(self) -> Self:
        """Async enter.

        Returns
        -------
            The P1 Monitor object.

        """
        return self

    async def __aexit__(self, *_exc_info: object) -> None:
        """Async exit.

        Args:
        ----
            _exc_info: Exec type.

        """
        await self.close()



"""Test the models."""

import pytest
from aiohttp import ClientSession
from aresponses import ResponsesMockServer
from syrupy.assertion import SnapshotAssertion

from p1monitor import (
    P1Monitor,
    P1MonitorConnectionError,
    P1MonitorNoDataError,
    Phases,
    Settings,
    SmartMeter,
    WaterMeter,
)

from . import load_fixtures


async def test_smartmeter(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    p1monitor_client: P1Monitor,
) -> None:
    """Test request from a P1 Monitor device - SmartMeter object."""
    aresponses.add(
        "192.168.1.2",
        "/api/v1/smartmeter",
        "GET",
        aresponses.Response(
            text=load_fixtures("smartmeter.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    smartmeter: SmartMeter = await p1monitor_client.smartmeter()
    assert smartmeter == snapshot


async def test_phases(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    p1monitor_client: P1Monitor,
) -> None:
    """Test request from a P1 Monitor device - Phases object."""
    aresponses.add(
        "192.168.1.2",
        "/api/v1/status",
        "GET",
        aresponses.Response(
            text=load_fixtures("phases.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    phases: Phases = await p1monitor_client.phases()
    assert phases == snapshot


async def test_watermeter(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    p1monitor_client: P1Monitor,
) -> None:
    """Test request from a P1 Monitor device - WaterMeter object."""
    aresponses.add(
        "192.168.1.2",
        "/api/v2/watermeter/day",
        "GET",
        aresponses.Response(
            text=load_fixtures("watermeter.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    watermeter: WaterMeter = await p1monitor_client.watermeter()
    assert watermeter == snapshot


async def test_no_watermeter_data_new(aresponses: ResponsesMockServer) -> None:
    """Test no WaterMeter data from P1 Monitor device."""
    aresponses.add(
        "192.168.1.2",
        "/api/v2/watermeter/day",
        "GET",
        aresponses.Response(
            text=load_fixtures("no_data.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )

    async with ClientSession() as session:
        client = P1Monitor(host="192.168.1.2", session=session)
        with pytest.raises(P1MonitorNoDataError):
            await client.watermeter()


async def test_no_watermeter_data_old(aresponses: ResponsesMockServer) -> None:
    """Test no WaterMeter data from P1 Monitor device."""
    aresponses.add(
        "192.168.1.2",
        "/api/v2/watermeter/day",
        "GET",
        aresponses.Response(
            status=404,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )

    async with ClientSession() as session:
        client = P1Monitor(host="192.168.1.2", session=session)
        with pytest.raises(P1MonitorConnectionError):
            await client.watermeter()


async def test_settings(
    aresponses: ResponsesMockServer,
    snapshot: SnapshotAssertion,
    p1monitor_client: P1Monitor,
) -> None:
    """Test request from a P1 Monitor device - Settings object."""
    aresponses.add(
        "192.168.1.2",
        "/api/v1/configuration",
        "GET",
        aresponses.Response(
            text=load_fixtures("settings.json"),
            status=200,
            headers={"Content-Type": "application/json; charset=utf-8"},
        ),
    )
    settings: Settings = await p1monitor_client.settings()
    assert settings == snapshot
